@@ -17,13 +17,13 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-import sys, os, locale
+import sys, os, locale, gettext
 from optparse import OptionParser
 import genshi.core
 import ConfigParser
 
 import lazygal
-from lazygal.generators import Album
+from lazygal.generators import Album, SOURCEDIR_CONFIGFILE
 
 CONFIGFILE = '~/.lazygal/config'
 CONFIGDEFAULTS = {
@@ -34,8 +34,20 @@ CONFIGDEFAULTS = {
     'thumbnail-size': '150x113',
     'quality': '85',
     'template-vars': '',
+    'thumbs-per-page': '0',
+    'webalbumpic-bg': 'transparent',
 }
 
+# i18n
+LOCALES_PATH = os.path.normpath(os.path.join(os.path.dirname(__file__),
+                                             'build', 'mo'))
+if not os.path.exists(LOCALES_PATH):
+    LOCALES_PATH = '/usr/share/locale'
+gettext.install('lazygal', LOCALES_PATH, unicode=1)
+
+locale.setlocale(locale.LC_ALL, locale.getdefaultlocale())
+
+
 # Read configuration file
 config = ConfigParser.ConfigParser(defaults = CONFIGDEFAULTS)
 # The following will hold until the config file has more than one section.
@@ -43,88 +55,112 @@ config = ConfigParser.ConfigParser(defaults = CONFIGDEFAULTS)
 config.add_section('lazygal')
 config.read(os.path.expanduser(CONFIGFILE))
 
-usage = "usage: %prog [options] albumdir"
+usage = _("usage: %prog [options] albumdir")
 parser = OptionParser(usage=usage)
+
+# The help option must be changed to comply with i18n.
+parser.get_option('-h').help = _("Show this help message and exit.")
+
 parser.add_option("", "--quiet",
                   action="store_true",
                   dest="quiet", default=False,
-                  help="Don't output anything except for errors.")
+                  help=_("Don't output anything except for errors."))
 parser.add_option("", "--debug",
                   action="store_true",
                   dest="debug", default=False,
-                  help="Output everything that lazygal is doing.")
+                  help=_("Output everything that lazygal is doing."))
 parser.add_option("-o", "--output-directory",
                   action="store", type="string",
                   dest="dest_dir", default=".",
-                  help="Directory where web pages, slides and thumbs will be written (default is current directory).")
+                  help=_("Directory where web pages, slides and thumbs will be written (default is current directory)."))
 parser.add_option("-t", "--theme",
                   action="store", type="string",
                   dest="theme",
                   default=config.get('lazygal', 'theme'),
-                  help="Theme name (looked up in theme directory) or theme full path.")
+                  help=_("Theme name (looked up in theme directory) or theme full path."))
 parser.add_option("", "--default-style",
                   action="store", type="string",
                   dest="default_style",
                   default=config.get('lazygal', 'default-style'),
-                  help="Default style to apply to the theme.")
+                  help=_("Default style to apply to the theme."))
 parser.add_option("", "--template-vars",
                   action="store", type="string",
                   dest="tpl_vars",
                   default=config.get('lazygal', 'template-vars'),
-                  help="Common variables to load all templates with.")
+                  help=_("Common variables to load all templates with."))
 parser.add_option("", "--clean-destination",
                   action="store_true",
                   dest="clean_dest",
                   default=config.getboolean('lazygal', 'clean-destination'),
-                  help="Clean destination directory of files that should not be there.")
+                  help=_("Clean destination directory of files that should not be there."))
 parser.add_option("-v", "--version",
                   action="store_true",
                   dest="show_version", default=False,
-                  help="Display program version.")
+                  help=_("Display program version."))
 parser.add_option("", "--check-all-dirs",
                   action="store_true",
                   dest="check_all_dirs", default=False,
-                  help="Exhaustively go through all directories regardless of source modification time.")
+                  help=_("Exhaustively go through all directories regardless of source modification time."))
 parser.add_option("-s", "--image-size",
                   action="store", type="string",
                   dest="image_size",
                   default=config.get('lazygal', 'image-size'),
-                  help="Size of images, define as <name>=<x>x<y>,..., eg. small=800x600,medium=1024x768.")
+                  help=_("Size of images, define as <name>=<x>x<y>,..., eg. small=800x600,medium=1024x768."))
 parser.add_option("-T", "--thumbnail-size",
                   action="store", type="string",
                   dest="thumbnail_size",
                   default=config.get('lazygal', 'thumbnail-size'),
-                  help="Size of thumbnails, define as <x>x<y>, eg. 150x113.")
+                  help=_("Size of thumbnails, define as <x>x<y>, eg. 150x113."))
 parser.add_option("-q", "--quality",
                   action="store", type="int",
                   dest="quality",
                   default=config.get('lazygal', 'quality'),
-                  help="Quality of generated JPEG images (default is 85).")
+                  help=_("Quality of generated JPEG images (default is 85)."))
 parser.add_option("-O", "--original",
                   action="store_true",
                   dest="original", default=False,
-                  help="Include original photos in output.")
+                  help=_("Include original photos in output."))
 parser.add_option("", "--puburl",
                   action="store", type="string",
                   dest="pub_url",
-                  help="Publication URL (only usefull for feed generation).")
+                  help=_("Publication URL (only usefull for feed generation)."))
 parser.add_option("-m", "--generate-metadata",
                   action="store_true",
                   dest="metadata", default=False,
-                  help="Generate metadata description files where they don't exist.")
+                  help=_("Generate metadata description files where they don't exist."))
+parser.add_option("-n", "--thumbs-per-page",
+                  action="store", type="int",
+                  dest="thumbs_per_page",
+                  default=config.get('lazygal', 'thumbs-per-page'),
+                  help=_("Maximum number of thumbs per index page. This enables index pagination (0 is unlimited)."))
+parser.add_option("-z", "--make-dir-zip",
+                  action="store_true",
+                  dest="dirzip", default=False,
+                  help=_("Make a zip archive of original pictures for each directory."))
+parser.add_option("", "--webalbum-pic-bg",
+                  action="store", type="string",
+                  dest="webalbumpic_bg",
+                  default=config.get('lazygal', 'webalbumpic-bg'),
+                  help=_("Webalbum picture background color. Default is transparent, and implies the PNG format. Any other value, e.g. red, white, blue, uses JPEG."))
 (options, args) = parser.parse_args()
 
 if options.show_version:
-    print 'lazygal version %s' % lazygal.__version__
+    print _('lazygal version %s') % lazygal.__version__
     sys.exit(0)
 
 if len(args) != 1:
     parser.print_help()
-    sys.exit("Bad command line.")
+    sys.exit(_("Bad command line."))
 
 source_dir = args[0]
 if not os.path.isdir(source_dir):
-    sys.exit("Directory %s does not exist.", source_dir)
+    print _("Directory %s does not exist.") % source_dir
+    sys.exit(1)
+
+# Load a config file in the source_dir root
+sourcedir_configfile = os.path.join(source_dir, SOURCEDIR_CONFIGFILE)
+if os.path.isfile(sourcedir_configfile):
+    config.read(sourcedir_configfile)
 
 sizes = []
 size_defs = options.image_size.split(',')
@@ -136,7 +172,9 @@ for single_def in size_defs:
 x, y = options.thumbnail_size.split('x')
 thumbnail = (int(x), int(y))
 
-album = Album(source_dir, thumbnail, sizes, quality=options.quality)
+album = Album(source_dir, thumbnail, sizes, quality=options.quality,
+              thumbs_per_page=options.thumbs_per_page,
+              dirzip=options.dirzip)
 
 if options.tpl_vars or config.has_section('template-vars'):
     tpl_vars = {}
@@ -155,6 +193,7 @@ if options.tpl_vars or config.has_section('template-vars'):
 
 album.set_theme(options.theme, options.default_style)
 album.set_original(options.original)
+album.set_webalbumpic(bg=options.webalbumpic_bg)
 
 log_level = None
 if options.quiet:
