@@ -19,8 +19,7 @@ import os, re
 
 
 __all__ = [
-        'EXIF',
-        'lazygal',
+        'generators',
         ]
 
 def get_darcs_lastdate():
@@ -48,6 +47,6 @@ def get_darcs_lastdate():
         return ''
 
 
-__version__ = '0.3.1' + get_darcs_lastdate()
+__version__ = '0.4' + get_darcs_lastdate()
 
 # vim: ts=4 sw=4 expandtab
