@@ -16,8 +16,13 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 import os, glob, sys, string
-import Image
+import zipfile
+import locale
 
+import Image
+import genshi
+
+import __init__
 from lazygal import make, sourcetree, tpl, metadata, feeds, eyecandy
 
 
@@ -25,13 +30,15 @@ DATAPATH = os.path.normpath(os.path.join(os.path.dirname(__file__), '..'))
 if not os.path.exists(os.path.join(DATAPATH, 'themes')):
     DATAPATH = os.path.join(sys.exec_prefix, 'share', 'lazygal')
     if not os.path.exists(os.path.join(DATAPATH, 'themes')):
-        print 'Could not find themes dir, check your installation!'
+        print _('Could not find themes dir, check your installation!')
 
 THEME_DIR = os.path.join(DATAPATH, 'themes')
 USER_THEME_DIR = os.path.expanduser(os.path.join('~', '.lazygal', 'themes'))
 THEME_SHARED_FILE_PREFIX = 'SHARED_'
 DEST_SHARED_DIRECTORY_NAME = 'shared'
 
+SOURCEDIR_CONFIGFILE = '.lazygal'
+
 
 class ImageOriginal(make.FileCopy):
 
@@ -84,12 +91,13 @@ class ImageOtherSize(make.FileMakeObject):
 
 class WebalbumPicture(make.FileMakeObject):
 
-    FILENAME = 'index.png'
+    BASEFILENAME = 'index'
 
     def __init__(self, lightdir):
-        self.path = os.path.join(lightdir.path, WebalbumPicture.FILENAME)
-        make.FileMakeObject.__init__(self, self.path)
         self.album = lightdir.album
+        self.path = os.path.join(lightdir.path,
+                                 self.album.get_webalbumpic_filename())
+        make.FileMakeObject.__init__(self, self.path)
 
         self.add_dependency(lightdir.source_dir)
 
@@ -107,14 +115,48 @@ class WebalbumPicture(make.FileMakeObject):
             md_dirpic_thumb = os.path.join(lightdir.path, md_dirpic_thumb)
         else:
             md_dirpic_thumb = None
-        self.dirpic = eyecandy.PictureMess(pics, md_dirpic_thumb)
+        self.dirpic = eyecandy.PictureMess(pics, md_dirpic_thumb,
+                                           bg=self.album.webalbumpic_bg)
 
     def build(self):
-        self.album.log("  DIRPIC %s" % os.path.basename(self.path), 'info')
+        self.album.log(_("  DIRPIC %s") % os.path.basename(self.path), 'info')
         self.album.log("(%s)" % self.path)
         self.dirpic.write(self.path)
 
 
+class WebalbumArchive(make.FileMakeObject):
+
+    def __init__(self, lightdir):
+        self.path = os.path.join(lightdir.path,
+                                 lightdir.source_dir.name + '.zip')
+        make.FileMakeObject.__init__(self, self.path)
+
+        self.dir = lightdir
+        self.album = self.dir.album
+
+        self.dir.dirzip = self
+
+        self.add_dependency(self.dir.source_dir)
+
+        self.pics = map(lambda x: os.path.join(self.dir.source_dir.path, x),
+                        self.dir.images_names)
+        for pic in self.pics:
+            self.add_file_dependency(pic)
+
+    def build(self):
+        self.album.log(_("  ZIP %s") % os.path.basename(self.path), 'info')
+        self.album.log("(%s)" % self.path)
+
+        archive = zipfile.ZipFile(self.path, mode='w')
+        for pic in self.pics:
+            inzip_filename = os.path.join(self.dir.source_dir.name,
+                                          os.path.basename(pic))
+            # zipfile dislikes unicode
+            inzip_fn = inzip_filename.encode(locale.getpreferredencoding())
+            archive.write(pic, inzip_fn)
+        archive.close()
+
+
 class WebalbumPage(make.FileMakeObject):
 
     def __init__(self, dir, size_name, base_name):
@@ -165,6 +207,9 @@ class WebalbumPage(make.FileMakeObject):
     def _add_size_qualifier(self, path, size_name):
         return self.dir.album._add_size_qualifier(path, size_name)
 
+    def _do_not_escape(self, value):
+        return genshi.core.Markup(value)
+
 
 class WebalbumBrowsePage(WebalbumPage):
 
@@ -191,7 +236,7 @@ class WebalbumBrowsePage(WebalbumPage):
                 self.add_dependency(ImageOtherSize(self.dir, prevnext, 'thumb'))
 
     def build(self):
-        self.dir.album.log("  XHTML %s" % os.path.basename(self.page_path),
+        self.dir.album.log(_("  XHTML %s") % os.path.basename(self.page_path),
                            'info')
         self.dir.album.log("(%s)" % self.page_path)
 
@@ -209,7 +254,10 @@ class WebalbumBrowsePage(WebalbumPage):
             tpl_values['img_height'] = self.image.get_size(browse_image_path)
 
         img_date = self.image.get_date_taken()
-        tpl_values['image_date'] = img_date.strftime("on %d/%m/%Y at %H:%M")
+        # strftime does not work with utf-8...
+        time_format = _("on %d/%m/%Y at %H:%M").encode(locale.getpreferredencoding())
+        time_str = img_date.strftime(time_format)
+        tpl_values['image_date'] = time_str.decode(locale.getpreferredencoding())
 
         tpl_values['prev_link'] =\
             self._gen_other_img_link(self.image.previous_image)
@@ -237,18 +285,37 @@ class WebalbumBrowsePage(WebalbumPage):
 
 class WebalbumIndexPage(WebalbumPage):
 
-    def __init__(self, dir, size_name):
-        WebalbumPage.__init__(self, dir, size_name, 'index')
+    FILENAME_BASE_STRING = 'index'
+
+    def __init__(self, dir, size_name, page_number=0):
+        page_paginated_name = self._get_paginated_name(page_number)
+        WebalbumPage.__init__(self, dir, size_name, page_paginated_name)
+
+        self.page_number = page_number
+
+        if self.dir.album.thumbs_per_page == 0:
+            # No pagination
+            self.images = dir.images
+            self.dirnames = dir.source_dir.dirnames
+            self.subdirs = dir.subdirs
+        else:
+            step = self.page_number * self.dir.album.thumbs_per_page
+            self.images = dir.images[step:step+self.dir.album.thumbs_per_page]
+
+            # subgal links only for first page
+            if self.page_number == 0:
+                self.dirnames = dir.source_dir.dirnames
+                self.subdirs = dir.subdirs
+            else:
+                self.dirnames = []
+                self.subdirs = []
 
-        self.images = dir.images
         for image in self.images:
             thumb_dep = ImageOtherSize(self.dir, image, 'thumb')
             self.add_dependency(thumb_dep)
             image_dep = WebalbumBrowsePage(self.dir, size_name, image)
             self.add_dependency(image_dep)
 
-        self.dirnames = dir.source_dir.dirnames
-        self.subdirs = dir.subdirs
         for subdir in self.subdirs:
             self.add_dependency(subdir)
             self.add_dependency(subdir.metadata)
@@ -258,14 +325,42 @@ class WebalbumIndexPage(WebalbumPage):
 
         self.add_dependency(dir.metadata)
 
+    def _get_paginated_name(self, page_number=None):
+        if page_number == None:
+            page_number = self.page_number
+        assert page_number != None
+
+        if page_number < 1:
+            return WebalbumIndexPage.FILENAME_BASE_STRING
+        else:
+            return '_'.join([WebalbumIndexPage.FILENAME_BASE_STRING,
+                             str(page_number)])
+
+    def _get_onum_links(self):
+        onum_index_links = []
+        for onum in range(0, self.dir.how_many_pages):
+            onum_info = {}
+            if onum == self.page_number:
+                # No link if we're on the current page
+                onum_info['name'] = onum
+            else:
+                onum_info['name'] = onum
+                filename = self._get_paginated_name(onum)
+                onum_info['link'] = self._add_size_qualifier(filename + '.html',
+                                                             self.size_name)
+            onum_index_links.append(onum_info)
+
+        return onum_index_links
+
     def build(self):
-        self.dir.album.log("  XHTML %s" % os.path.basename(self.page_path),
+        self.dir.album.log(_("  XHTML %s") % os.path.basename(self.page_path),
                            'info')
         self.dir.album.log("(%s)" % self.page_path)
 
         values = {}
 
-        values['osize_index_links'] = self._get_osize_links('index')
+        values['osize_index_links'] = self._get_osize_links(self._get_paginated_name())
+        values['onum_index_links'] = self._get_onum_links()
 
         subgal_links = []
         for subdir in self.subdirs:
@@ -273,11 +368,17 @@ class WebalbumIndexPage(WebalbumPage):
                         'link': subdir.source_dir.name + '/'}
             dir_info.update(self.dir.metadata.get(subdir.source_dir.name))
             dir_info['album_picture'] = os.path.join(subdir.source_dir.name,
-                                                     WebalbumPicture.FILENAME)
+                                     self.dir.album.get_webalbumpic_filename())
+            if 'album_description' in dir_info.keys():
+                dir_info['album_description'] =\
+                             self._do_not_escape(dir_info['album_description'])
             subgal_links.append(dir_info)
         values['subgal_links'] = subgal_links
         if self.dir.metadata:
             values.update(self.dir.metadata.get())
+            if 'album_description' in values.keys():
+                values['album_description'] =\
+                             self._do_not_escape(values['album_description'])
 
         values['images'] = map(self._gen_other_img_link, self.images)
 
@@ -285,6 +386,9 @@ class WebalbumIndexPage(WebalbumPage):
         values['rel_path'] = self.dir.source_dir.strip_root()
         values['title'] = self.dir.human_name
 
+        if self.dir.album.dirzip and self.dir.dirzip:
+            values['dirzip'] = os.path.basename(self.dir.dirzip.path)
+
         self.page_template.dump(values, self.page_path)
 
 
@@ -305,11 +409,13 @@ class LightWebalbumDir(make.FileMakeObject):
         for filename in self.source_dir.filenames:
             if self.album._is_ext_supported(filename):
                 self.images_names.append(filename)
-            elif not filename == metadata.MATEW_METADATA:
-                self.album.log("  Ignoring %s, format not supported."\
-                               % filename, 'info')
-                self.album.log("(%s)" % os.path.join(self.source_dir.path,
-                                                     filename))
+            elif filename not in (metadata.MATEW_METADATA,
+                                  SOURCEDIR_CONFIGFILE):
+                if self.__class__.__name__ != 'LightWebalbumDir':
+                    self.album.log(_("  Ignoring %s, format not supported.")\
+                                   % filename, 'info')
+                    self.album.log("(%s)" % os.path.join(self.source_dir.path,
+                                                         filename))
 
         self.image_count = len(self.images_names)
         self.subgal_count = len(self.subdirs)
@@ -330,6 +436,8 @@ class LightWebalbumDir(make.FileMakeObject):
         else:
             self.album_picture = None
 
+        self.dirzip = None
+
     def get_all_images_paths(self):
         all_images_paths = map(lambda fn: os.path.join(self.path, fn),
                                self.images_names)
@@ -355,7 +463,7 @@ class WebalbumDir(LightWebalbumDir):
 
         # Create the directory if it does not exist
         if not os.path.isdir(self.path):
-            self.album.log("  MKDIR %%WEBALBUMROOT%%/%s"\
+            self.album.log(_("  MKDIR %%WEBALBUMROOT%%/%s")\
                            % self.source_dir.strip_root(), 'info')
             self.album.log("(%s)" % self.path)
             os.makedirs(self.path, mode = 0755)
@@ -365,10 +473,27 @@ class WebalbumDir(LightWebalbumDir):
                                       album),
                                       self.images_names)
 
+        self.how_many_pages = None
+        if self.album.thumbs_per_page == 0\
+        or self.image_count <= self.album.thumbs_per_page:
+            # No pagination (requested or needed)
+            self.how_many_pages = 1
+        else:
+            # Pagination requested and needed
+            how_many_pages = self.image_count / self.album.thumbs_per_page
+            if self.image_count % self.album.thumbs_per_page > 0:
+                how_many_pages = how_many_pages + 1
+            assert how_many_pages > 1
+            self.how_many_pages = how_many_pages
+
         for size_name in self.album.browse_sizes.keys():
-            self.add_dependency(WebalbumIndexPage(self, size_name))
+            for page_number in range(0, self.how_many_pages):
+                self.add_dependency(WebalbumIndexPage(self, size_name,
+                                                      page_number))
 
         self.add_dependency(WebalbumPicture(self))
+        if self.album.dirzip and self.image_count > 1:
+            self.add_dependency(WebalbumArchive(self))
 
     def prepare(self):
         self.images.sort(lambda x, y: x.compare_date_taken(y))
@@ -394,6 +519,9 @@ class WebalbumDir(LightWebalbumDir):
 
         for dest_file in os.listdir(self.path):
             dest_file = os.path.join(self.path, dest_file)
+            if not isinstance(dest_file, unicode):
+                # No clue why this happens, but it happens!
+                dest_file = dest_file.decode(sys.getfilesystemencoding())
             if dest_file not in self.output_items and\
                dest_file not in self.source_dir.dirnames:
                 text = ''
@@ -403,8 +531,8 @@ class WebalbumDir(LightWebalbumDir):
                         os.unlink(rmv_candidate)
                         text = ""
                     else:
-                        text = "you should "
-                    self.album.log("  %sRM %s" % (text, dest_file), 'info')
+                        text = _("you should ")
+                    self.album.log(_("  %sRM %s") % (text, dest_file), 'info')
 
     def make(self, force=False):
         make.FileMakeObject.make(self, force)
@@ -448,7 +576,7 @@ class WebalbumFeed(make.FileMakeObject):
 
         desc_values = {}
         desc_values['album_pic_path'] = os.path.join(url,
-                                                     WebalbumPicture.FILENAME)
+                                          self.album.get_webalbumpic_filename())
         desc_values['subgal_count'] = webalbumdir.subgal_count
         desc_values['picture_count'] = webalbumdir.image_count
         desc_values['desc'] = webalbumdir.desc
@@ -458,7 +586,7 @@ class WebalbumFeed(make.FileMakeObject):
                             webalbumdir.source_dir.get_mtime())
 
     def build(self):
-        self.album.log("FEED %s" % os.path.basename(self.path), 'info')
+        self.album.log(_("FEED %s") % os.path.basename(self.path), 'info')
         self.album.log("(%s)" % self.path)
         self.feed.dump(self.path)
 
@@ -470,7 +598,7 @@ class SharedFileCopy(make.FileCopy):
         self.album = album
 
     def build(self):
-        self.album.log("CP %%SHAREDDIR%%/%s" %\
+        self.album.log(_("CP %%SHAREDDIR%%/%s") %\
                        os.path.basename(self.dst), 'info')
         self.album.log("(%s)" % self.dst)
         make.FileCopy.build(self)
@@ -487,13 +615,13 @@ class SharedFileTemplate(make.FileMakeObject):
         if ext.startswith('.t'):
             self.path = filename + '.' + ext[2:]
         else:
-            raise ValueError('We have a template with an extension that does not start with a t. Abording.')
+            raise ValueError(_('We have a template with an extension that does not start with a t. Abording.'))
 
         make.FileMakeObject.__init__(self, self.path)
         self.add_file_dependency(shared_tpl_name)
 
     def build(self):
-        self.album.log("TPL %%SHAREDDIR%%/%s"\
+        self.album.log(_("TPL %%SHAREDDIR%%/%s")\
                        % os.path.basename(self.path), 'info')
         self.album.log("(%s)" % self.path)
         self.tpl.dump({}, self.path)
@@ -507,7 +635,7 @@ class SharedFiles(make.FileSimpleDependency):
 
         # Create the shared files directory if it does not exist
         if not os.path.isdir(self.path):
-            album.log("MKDIR %SHAREDDIR%", 'info')
+            album.log(_("MKDIR %SHAREDDIR%"), 'info')
             album.log("(%s)" % self.path)
             os.makedirs(self.path, mode = 0755)
 
@@ -531,7 +659,7 @@ class SharedFiles(make.FileSimpleDependency):
 class Album:
 
     def __init__(self, source_dir, thumb_size, browse_sizes,
-                 quality=85, debug=False):
+                 quality=85, thumbs_per_page=0, dirzip=False):
         self.set_logging()
 
         self.source_dir = os.path.abspath(source_dir)
@@ -545,6 +673,8 @@ class Album:
         self.tpl_loader = None
         self.tpl_vars = {}
         self.original = False
+        self.thumbs_per_page = thumbs_per_page
+        self.dirzip = dirzip
 
     def set_theme(self, theme='default', default_style=None):
         self.theme = theme
@@ -556,9 +686,9 @@ class Album:
             # Fallback to system themes
             self.tpl_dir = os.path.join(THEME_DIR, self.theme)
             if not os.path.exists(self.tpl_dir):
-                raise ValueError('Theme %s not found' % self.theme)
+                raise ValueError(_('Theme %s not found') % self.theme)
 
-        self.tpl_loader = tpl.TplFactory([self.tpl_dir])
+        self.tpl_loader = tpl.TplFactory(self.tpl_dir)
         self.tpl_vars.update({'styles' : self.get_avail_styles(self.theme,
                                                                default_style)})
         self.set_tpl_vars()
@@ -595,6 +725,16 @@ class Album:
     def set_original(self, original = False):
         self.original = original
 
+    def set_webalbumpic(self, bg='transparent'):
+        self.webalbumpic_bg = bg
+
+    def get_webalbumpic_filename(self):
+        if self.webalbumpic_bg == 'transparent':
+            ext = '.png' # JPEG does not have an alpha channel
+        else:
+            ext = '.jpg'
+        return WebalbumPicture.BASEFILENAME + ext
+
     log_levels = ['debug', 'info', 'error']
 
     def set_logging(self, level='info', outpipe=sys.stdout,
@@ -641,18 +781,19 @@ class Album:
         '''
         Generate default metada files if no exists.
         '''
-        self.log("Generating metadata in %s" % self.source_dir)
+        self.log(_("Generating metadata in %s") % self.source_dir)
 
         for root, dirnames, filenames in os.walk(self.source_dir):
             dir = sourcetree.Directory(root, dirnames, filenames, self)
-            self.log("[Entering %%ALBUMROOT%%/%s]" % dir.strip_root(), 'info')
+            self.log(_("[Entering %%ALBUMROOT%%/%s]") % dir.strip_root(),
+                     'info')
             self.log("(%s)" % dir.path)
 
             md = metadata.DirectoryMetadata(dir)
 
             md_data = md.get()
             if 'album_description' in md_data.keys() or 'album_name' in md_data.keys():
-                self.log("  SKIPPED because metadata exists.")
+                self.log(_("  SKIPPED because metadata exists."))
             else:
                 md.generate()
 
@@ -661,9 +802,9 @@ class Album:
         sane_dest_dir = os.path.abspath(dest_dir)
 
         if self.is_in_sourcetree(sane_dest_dir):
-            raise ValueError("Fatal error, web gallery directory is within source tree.")
+            raise ValueError(_("Fatal error, web gallery directory is within source tree."))
 
-        self.log("Generating to %s" % sane_dest_dir)
+        self.log(_("Generating to %s") % sane_dest_dir)
 
         if pub_url and feeds.HAVE_ETREE:
             feed = WebalbumFeed(self, sane_dest_dir, pub_url)
@@ -677,10 +818,11 @@ class Album:
             dir = sourcetree.Directory(root, dirnames, filenames, self)
 
             if dir.should_be_skipped():
-                self.log("(%s) has been skipped" % dir.path)
+                self.log(_("(%s) has been skipped") % dir.path)
                 continue
 
-            self.log("[Entering %%ALBUMROOT%%/%s]" % dir.strip_root(), 'info')
+            self.log(_("[Entering %%ALBUMROOT%%/%s]") % dir.strip_root(),
+                     'info')
             self.log("(%s)" % dir.path)
 
             if dir_heap.has_key(root):
@@ -711,7 +853,7 @@ class Album:
             if destgal.needs_build() or check_all_dirs:
                 destgal.make()
             else:
-                self.log("  SKIPPED because of mtime, touch source or use --check-all-dirs to override.")
+                self.log(_("  SKIPPED because of mtime, touch source or use --check-all-dirs to override."))
 
         if feed:
             feed.make()
