@@ -15,7 +15,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-import os, sys, datetime
+import os, sys, datetime, locale
 import pyexiv2, Image
 
 from lazygal import make
@@ -148,7 +148,7 @@ class ExifTags(pyexiv2.Image):
         is not found.
         '''
         try:
-            return str(self[name])
+            return str(self[name]).strip(' ')
         except (IndexError, ValueError, KeyError):
             return ''
 
@@ -176,6 +176,12 @@ class ExifTags(pyexiv2.Image):
         else:
             return float(val.numerator) / float(val.denominator)
 
+    def __fallback_to_encoding(self, encoded_string, encoding='utf-8'):
+        try:
+            return encoded_string.decode(encoding)
+        except UnicodeDecodeError:
+            return encoded_string.decode(encoding, 'replace')
+
     def get_jpeg_comment(self):
         '''
         Reads JPEG comment field, returns empty string if key is not
@@ -183,19 +189,58 @@ class ExifTags(pyexiv2.Image):
         '''
         im = Image.open(self.image_path)
         try:
-            return im.app['COM']
+            return self.__fallback_to_encoding(im.app['COM'])
         except KeyError:
             return ''
 
+    def get_exif_usercomment(self):
+        ret = self.get_exif_string('Exif.Photo.UserComment')
+        # This field can contain charset information
+        # FIXME : All this stuff should really go in pyexiv2.
+        if ret.startswith('charset='):
+            tokens = ret.split(' ')
+            csetfield = tokens[0]
+            text = ' '.join(tokens[1:])
+            ignore, cset = csetfield.split('=')
+            cset = cset.strip('"')
+        else:
+            cset = None
+            text = ret
+
+        if cset == 'Unicode':
+            im = Image.open(self.image_path)
+            endian = im.app['APP1'][6:8]
+            if endian == 'MM':
+                encoding = 'utf-16be'
+            elif endian == 'II':
+                encoding = 'utf-16le'
+            else:
+                return ''
+
+        elif cset == 'Ascii':
+            encoding = 'ascii'
+        elif cset == 'Jis':
+            encoding = 'shift_jis'
+        else:
+            # Fallback to utf-8 as this is mostly the default for Linux
+            # distributions.
+            encoding = 'utf-8'
+
+        return self.__fallback_to_encoding(text, encoding).strip('\0')
+
     def get_comment(self):
         try:
-            ret = self.get_exif_string('Exif.Photo.UserComment')
+            ret = self.get_exif_usercomment()
             if ret == '':
                 raise ValueError
-            else:
-                return ret
         except (ValueError, KeyError):
-            return self.get_jpeg_comment()
+            try:
+                ret = self.get_exif_string('Exif.Image.ImageDescription')
+                if ret == '':
+                    raise ValueError
+            except (ValueError, KeyError):
+                ret = self.get_jpeg_comment()
+        return ret
 
     def get_flash(self):
         return self.get_exif_value('Exif.Photo.Flash')
@@ -233,7 +278,7 @@ class ExifTags(pyexiv2.Image):
 
         flen35 = self.get_exif_float('Exif.Photo.FocalLengthIn35mmFilm')
         if flen35 != '':
-            flen += ' (35 mm equivalent: %s mm)' % flen35
+            flen += _(' (35 mm equivalent: %s mm)') % flen35
             return flen
 
         try:
@@ -255,8 +300,9 @@ class ExifTags(pyexiv2.Image):
                 return ''
 
             foclength = self.get_exif_float_value('Exif.Photo.FocalLength')
+            lenstr = '%.01f' % (foclength / ccdwidth * 36 + 0.5)
 
-            flen += ' (35 mm equivalent: %.01f mm)' % (foclength / ccdwidth * 36 + 0.5)
+            flen += _(' (35 mm equivalent: %s mm)') % lenstr
         except (IndexError, KeyError):
             return flen
 
@@ -293,7 +339,7 @@ class DirectoryMetadata(make.FileSimpleDependency):
             path = os.path.join(self.directory_path, subdir, MATEW_METADATA)
 
         if path is None or not os.path.exists(path):
-            raise NoMetadata('Could not open metadata file (%s)' % path)
+            raise NoMetadata(_('Could not open metadata file %s') % path)
 
         f = file(path, 'r')
         for line in f:
@@ -311,7 +357,7 @@ class DirectoryMetadata(make.FileSimpleDependency):
                     if tag == 'album_picture':
                         if subdir is not None:
                             data = os.path.join(subdir, data)
-                    metadata[tag] = data.decode(sys.stdin.encoding)
+                    metadata[tag] = data.decode(locale.getpreferredencoding())
                     break
 
         return metadata
@@ -342,7 +388,7 @@ class DirectoryMetadata(make.FileSimpleDependency):
         Generates new metadata file with default values.
         '''
 
-        self.dir.album.log("GEN %s" %
+        self.dir.album.log(_("GEN %s") %
                 self.description_filename, 'info')
 
         md = self.get()
