@@ -19,28 +19,50 @@ import os, time
 from genshi.template import TemplateLoader, MarkupTemplate, TextTemplate
 import __init__
 
-class LazygalTemplate:
+
+class LazygalTemplate(object):
+
+    def __complement_values(self, values):
+        values.update(self.common_values)
+        values['gen_date'] = time.strftime("%c")
+        values['lazygal_version'] = __init__.__version__
+        return values
+
+    def __generate(self, values):
+        # We use here a 't' variable to hold all of the template values, which
+        # explains why every single template value is called with the '$t.'
+        # prefix.
+        #
+        # The cryptic _=_ is the way to pass the gettext translation function
+        # to the templates : the _() callable is assigned to the '_' keyword
+        # arg.
+        return self.generate(t=values, _=_)
+
+
+    def instanciate(self, values):
+        self.__complement_values(values)
+        # encoding=None gives us a unicode string instead of an utf-8 encoded
+        # string. This is because we are not out of lazygal yet.
+        return self.__generate(values).render(self.serialization_method,
+                                              encoding=None)
 
     def dump(self, values, dest):
-        values.update(self.common_values)
-        values['gen_date'] = time.strftime("%A, %d %B %Y %H:%M %Z")
-        values['lazygal_version'] = __init__.__version__
+        self.__complement_values(values)
 
         page = open(dest, 'w')
-        page.write(self.instanciate(values))
+        self.__generate(values).render(method=self.serialization_method,
+                                       out=page, encoding='utf-8')
         page.close()
 
 
 class XmlTemplate(LazygalTemplate, MarkupTemplate):
 
-    def instanciate(self, values):
-        return self.generate(t=values).render('xhtml')
+    serialization_method = 'xhtml'
 
 
 class PlainTemplate(LazygalTemplate, TextTemplate):
 
-    def instanciate(self, values):
-        return self.generate(t=values).render()
+    serialization_method = 'text'
 
 
 class TplFactory(TemplateLoader):
@@ -52,6 +74,13 @@ class TplFactory(TemplateLoader):
 
     common_values = None
 
+    def __init__(self, tpl_dir):
+        # We use lenient mode here because we want an easy way to check whether
+        # a template variable is defined, or the empty string, thus defined()
+        # will only work for the 'whether it is defined' part of the test.
+        super(TplFactory, self).__init__([tpl_dir],
+                                         variable_lookup='lenient')
+
     def set_common_values(self, values):
         self.common_values = values
 
@@ -69,7 +98,7 @@ class TplFactory(TemplateLoader):
                 tpl.common_values = {}
             return tpl
         else:
-            raise ValueError('Unknown template type')
+            raise ValueError(_('Unknown template type'))
 
 
 # vim: ts=4 sw=4 expandtab
